#version 330 core
#define NR_POINT_LIGHTS 4

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;


uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
};
uniform PointLight pointLights[NR_POINT_LIGHTS];

vec3 phongPointLight(vec3 n, vec3 v, vec3 texColor, vec3 lightDir, vec3 pointLcolor, vec3 attenuation) {
	float distance = length(lightDir);
	lightDir = normalize(lightDir);

	float att = 1.0f / (attenuation.x + distance * attenuation.y + distance * distance * attenuation.z);
	vec3 refelctionDir = reflect(-lightDir, n);

	vec3 diffuseColor = pointLcolor * texColor;
	diffuseColor = materialCoefficients.y * diffuseColor * max(0, dot(n, lightDir));
	vec3 specularColor = materialCoefficients.z * pointLcolor * pow(max(0, dot(refelctionDir, v)), specularAlpha);
	vec3 ambientColor = texColor * materialCoefficients.x;

	return ambientColor + diffuseColor*att + specularColor*att;
}

vec3 phongDirLight(vec3 n, vec3 v, vec3 texColor, vec3 lightDir, vec3 pointLcolor) {
	float distance = length(-lightDir);
	lightDir = normalize(-lightDir);

	vec3 refelctionDir = reflect(-lightDir, n);

	vec3 diffuseColor = pointLcolor * texColor;
	diffuseColor = materialCoefficients.y * diffuseColor * max(0, dot(n, lightDir));
	vec3 specularColor = materialCoefficients.z * pointLcolor * pow(max(0, dot(refelctionDir, v)), specularAlpha);
	vec3 ambientColor = texColor * materialCoefficients.x;

	return ambientColor + diffuseColor + specularColor;
}

void main()
{
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vec2(vert.uv)).rgb;
	vec3 lightColor;

	for(int i = 0; i< NR_POINT_LIGHTS; i++) {
		lightColor += phongPointLight(n,v,texColor, pointLights[i].position - vert.position_world, pointLights[i].color, pointLights[i].attenuation);
	}
	lightColor += phongDirLight(n,v,texColor, dirL.direction - vert.position_world, dirL.color)*materialCoefficients.y/2;
	
	color = vec4(lightColor, 0.5);
}