#version 330 core
layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

uniform float dt;

uniform int type;

float freq = 2.5;
float amp = 0.2;

float wave(float time, float freq, float amp) {
	float angle = (-time*2+freq*5.0);
	return sin(angle) * amp;
}

float waveNorm(float time, float freq, float amp) {
	float angle = (-time*2+freq*5.0);
	return -amp *freq * cos(angle);
}

void main()
{    
	float time = dt;
	float offset = -15;
	if(type==2) { time = -dt; offset = 15; }
	vert.uv = uv;

	vec3 newPosition = position;
	float normY = 0;

	float angle = (time + newPosition.x)*freq;

	newPosition.z = newPosition.z + sin(angle)*amp;
	normY = -amp * freq * cos(angle);

	float nx = newPosition.x + offset;
	float ny = newPosition.z;
	
	float freq = sqrt(nx*nx + ny*ny)/4;
	//newPosition.z += wave(dt, freq, amp);
	newPosition.z = position.z;
	normY += waveNorm(dt, freq, amp);

	vec3 newNormal = normalize(vec3(0.0,normY, 1.0));


	vec4 position_world_ = modelMatrix * vec4(newPosition, 1);
	vert.position_world = position_world_.xyz;
	vert.normal_world = normalMatrix * newNormal;

	gl_Position = viewProjMatrix * position_world_;
}