#version 330 core
layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;
uniform vec2 offset1;
uniform vec2 offset2;

uniform float dt;
uniform int type;

float amp = 0.2;
float freq = 2.5;

float wave(float time, float freq, float amp) {
	float angle = (-time*2+freq*5.0);
	return sin(angle) * amp;
}

float waveNorm(float time, float freq, float amp) {
	float angle = (-time*2+freq*5.0);
	return -amp *freq * cos(angle);
}

void main()
{    
	float time;
	if(type==1) time = dt;
	if(type==2) time = -dt;

	vert.uv = uv;
	float normY = 0;

	vec3 newPosition = position;


	float angle = (time + newPosition.x)*freq;

	newPosition.z = newPosition.z + sin(angle)*0.1;
	float normX = -amp * freq * cos(angle)/2;


	//first wave
	float nx = newPosition.x - offset1.x;
	float ny = newPosition.z + offset1.y;
	
	float freq = sqrt(nx*nx + ny*ny)/4;

	newPosition.z += wave(dt, freq, amp);
	normY += waveNorm(dt, freq, amp);

	//second wave
	nx = newPosition.x - offset2.x;
	ny = newPosition.z - offset2.y;	
	freq = sqrt(nx*nx + ny*ny)/4;

	newPosition.z = position.z;
	//newPosition.z += wave(dt, freq, amp);
	normY += waveNorm(dt, freq, amp);

	vec3 newNormal = normalize(vec3(normX, normY, 1.0));


	vec4 position_world_ = modelMatrix * vec4(newPosition, 1);
	vert.position_world = position_world_.xyz;
	vert.normal_world = normalMatrix * newNormal;

	gl_Position = viewProjMatrix * position_world_;
}